@@ -27,12 +27,14 @@
 #' one of "pseudo-random", "mlhs", "standard-halton", "scrambled-halton",
 #' "standard-sobol","scrambled-sobol".
 #' @param R An integer giving the number of draws to use. The default is 100.
-#' @param dudx A character string giving the name of the parameter in the
+#' @param dudx A character string giving the name of the prior in the
 #' denominator. Must be specified when optimizing for 'c-error'
 #' @param candidate_set A matrix or data frame in the "wide" format containing
 #' all permitted combinations of attributes. The default is NULL. If no
 #' candidate set is provided, then the full factorial subject to specified
-#' exclusions will be used.
+#' exclusions will be used. This is passed in as an object and not a character
+#' string. The candidate set will be expanded to include zero columns to
+#' consider alternative specific attributes.
 #' @param exclusions A list of exclusions Often this list will be pulled
 #' directly from the list of options or it is a modified list of exclusions
 #' @param control A list of control options
@@ -80,6 +82,7 @@ generate_design <- function(utility,
     return(design_object),
     add = TRUE
   )
+
   ## Match arguments ----
   design_object[["model"]] <- model <- match.arg(model)
   efficiency_criteria <- match.arg(efficiency_criteria, several.ok = TRUE)
@@ -131,52 +134,86 @@ generate_design <- function(utility,
   }
 
   ## Candidate set ----
-  cli_h2("Checking the candidate set and applying exclusions")
+  # We are only creating the candidate set if we are using a random or modified federov algorithm
+  if (!is.null(candidate_set) & algorithm == "rsc") stop("To use your supplied candidate set you must use either the 'random' or 'federov' algorithms.")
 
-  # If no candidate set is supplied generate full factorial if not run simple
-  # checks
-  if (is.null(candidate_set)) {
-    cli_alert_info("No candidate set supplied. The design will use the full factorial subject to supplied constraints.")
+  if (algorithm %in% c("random", "federov")) {
+    cli_h2("Checking the candidate set and applying exclusions")
 
-    candidate_set <- full_factorial(expand_attribute_levels(utility))
+    # If no candidate set is supplied generate full factorial if not run simple
+    # checks
+    if (is.null(candidate_set)) {
+      cli_alert_info("No candidate set supplied. The design will use the full factorial subject to supplied constraints.")
 
-    cli_alert_success("Full factorial created")
+      candidate_set <- full_factorial(expand_attribute_levels(utility))
 
-  } else {
-    stopifnot((is.matrix(candidate_set) || is.data.frame(candidate_set)))
+      cli_alert_success("Full factorial created")
+
+    } else {
+      stopifnot((is.matrix(candidate_set) || is.data.frame(candidate_set)))
+
+      candidate_names_idx <- names(candidate_set) %in% names(expand_attribute_levels(utility))
+
+      if (!all(candidate_names_idx)) {
+        problem <- paste(names(candidate_set)[!candidate_names_idx], collapse = ", ")
+
+        stop(
+          paste0("There are more attributes specified in the candidate set than are present in the utility functions. ", problem, " are not specified in the utility function. This could also be caused by a mismatch in the names. The names should be of the form <utility list element name>_<attribute name>. For example, in your case, they should correspond to: '" , paste(names(expand_attribute_levels(utility)), collapse = ", "), "' The candidate set must be supplied in 'wide' format.")
+        )
+      }
+
+      # Extract only the specified in the utility function to check
+      regex <- paste0("\\b", attribute_names(utility))
+      utility_attributes <- vector(mode = "list", length = length(utility))
+      for (i in seq_along(utility)) {
+        idx <- str_detect(utility[[i]], regex)
+        utility_attributes[[i]] <- paste(names(utility[i]), attribute_names(utility)[idx], sep = "_")
+      }
+
+      utility_attributes <- do.call(c, utility_attributes)
+
+      if (!all(utility_attributes %in% names(candidate_set))) {
+        stop(
+          paste0("Not all attributes specified in the utility functions are specified in the candidate set. This could be caused by a mismatch in the names. The names should be of the form <utility list element name>_<attribute name>. For example, in your case, they should correspond to: '" , paste(utility_attributes, collapse = ", "), "' The candidate set must be supplied in 'wide' format.")
+        )
+      }
+
+      candidate_levels <- apply(candidate_set, 2, function(x) unique(sort(x)))
+      utility_levels <- lapply(expand_attribute_levels(utility), as.numeric)
+
+      # Subset utility levels to only correspond to the ones specified
+      utility_levels <- utility_levels[utility_attributes]
+
+      if (!identical(candidate_levels[sort(names(candidate_levels))], utility_levels[sort(names(utility_levels))])) {
+        problem <- paste(names(which(mapply(function(x, y) length(x) - length(y), candidate_levels, utility_levels) != 0)), collapse = ", ")
+
+        stop(
+          paste0("The attribute levels determined by the supplied candidate set differs from those supplied in the utility function. Please ensure that all specified levels are present in the candidate set. The error occurs because there are too few/many levels for: ", problem, " in the candidate set")
+        )
+      }
+
+      # Expand candidate set to be square, i.e., fill in zero columns, for non-specified
+      expanded_names <- names(expand_attribute_levels(utility))
+      expr <- paste("cbind(candidate_set, ", paste(paste(expanded_names[!(expanded_names %in% utility_attributes)], 0, sep = " = "), collapse = ", "), ")")
+      candidate_set <- eval(parse(text = expr))
+      candidate_set <- candidate_set[, expanded_names]
 
-    if (!all(names(candidate_set) %in% names(expand_attribute_levels(utility)))) {
-      stop(
-        "Not all attributes specified in the utility functions are specified in
-        the candidate set. Make sure that all attributes are specified and that
-        the names used in the utility functions correspond to the column names
-        of the supplied candidate set. The candidate set must be supplied
-        in 'wide' format."
-      )
     }
 
-    if (!identical(apply(candidate_set, 2, function(x) unique(sort(x))), lapply(expand_attribute_levels(utility), as.numeric))) {
-      stop(
-        "The attribute levels determined by the supplied candidate set differs
-        from those supplied in the utility function. Please ensure that all
-        specified levels are present in the candidate set. "
-      )
+    # Apply the exclusions to the candidate set
+    candidate_set <- exclude(candidate_set, exclusions)
+
+    # Transform the candiate set such that attributes that are dummy coded
+    # are turned into factors. This ensures that we can use the model.matrix()
+    for (i in which(names(candidate_set) %in% dummy_names(utility))) {
+      candidate_set[, i] <- as.factor(candidate_set[, i])
     }
+
+    # candidate_set <- as.matrix(candidate_set)
+
+    cli_alert_success("All exclusions successfully applied")
   }
 
-  # Apply the exclusions to the candidate set
-  candidate_set <- exclude(candidate_set, exclusions)
-
-  # Transform the candiate set such that attributes that are dummy coded
-  # are turned into factors. This ensures that we can use the model.matrix()
-  for (i in which(names(candidate_set) %in% dummy_names(utility))) {
-    candidate_set[, i] <- as.factor(candidate_set[, i])
-  }
-
-  # candidate_set <- as.matrix(candidate_set)
-
-  cli_alert_success("All exclusions successfully applied")
-
   # Prepare the list of priors ----
   cli_h2("Preparing the list of priors")
 
@@ -235,6 +272,9 @@ generate_design <- function(utility,
 
   design_object[["time"]][["time_end"]] <- Sys.time()
 
+  # Turn the design object into a tibble to be tidyverse compatible
+  design_object[["design"]] <- tibble::as_tibble(design_object[["design"]])
+
   # Print final closing messages
   cat("\n\n")
   cli_h1("Cleaning up design environment")
